@@ -1,3 +1,8 @@
+# Copyright (c) 2017-2025 Soft8Soft, LLC. All rights reserved.
+#
+# Use of this software is subject to the terms of the Verge3D license
+# agreement provided at the time of installation or download, or which
+# otherwise accompanies this software in either electronic or hard copy form.
 import math
 
 import pluginUtils
@@ -62,6 +67,12 @@ ALPHA_MODE_MAP = [
     'ADD'
 ]
 
+TRANSPARENCY_HACK_MAP = [
+    'NONE',
+    'NEAREST_LAYER',
+    'TWO_PASS'
+]
+
 IBL_ENV_MAP = [
     'PMREM',
     'PROBE',
@@ -193,14 +204,10 @@ def initSettingsNode(nodeType):
     cmds.addExtension(nodeType=nodeType, longName='aoBentNormals', attributeType='bool',
                       defaultValue=False, niceName='Bent Normals')
 
-
-    # COMPAT: < 4.7, replaced by shadowFiltering with fewer options
     cmds.addExtension(nodeType=nodeType, longName='shadowFilteringType', attributeType='enum',
                       enumName='a:b:c:d:e:f', defaultValue=4)
-    # COMPAT: < 4.7, replaced by iblEnvironment with fewer options
     cmds.addExtension(nodeType=nodeType, longName='iblEnvironmentMode', attributeType='enum', enumName='a:b:c:d')
 
-
 def createSettingsNode():
     """
     instantiate v3dSettings node
@@ -209,16 +216,12 @@ def createSettingsNode():
     if len(cmds.ls(type='v3dSettings')) == 0:
         cmds.createNode('v3dSettings', shared=True, name='v3dSettings', skipSelect=True)
 
-
-    # COMPAT: <4.7, set iblEnvironment from iblEnvironmentMode
     iblEnvOld = cmds.getAttr('v3dSettings.iblEnvironmentMode')
     iblEnvNew = cmds.getAttr('v3dSettings.iblEnvironment')
 
-    # old set to non-default (non-pmrem), new set to default (pmrem)
     if iblEnvOld != 0 and iblEnvNew == 0:
         log.info('Refactoring global IBL environment mode setting')
 
-        # cubemap to pmrem
         if iblEnvOld == 1:
             iblEnvNew = 0
         else:
@@ -227,18 +230,12 @@ def createSettingsNode():
         cmds.setAttr('v3dSettings.iblEnvironmentMode', 0) # to default
         cmds.setAttr('v3dSettings.iblEnvironment', iblEnvNew)
 
-
-    # COMPAT: <4.7, set shadowFiltering from shadowFilteringType
     shadowFilteringOld = cmds.getAttr('v3dSettings.shadowFilteringType')
     shadowFilteringNew = cmds.getAttr('v3dSettings.shadowFiltering')
 
-    # old set to non-default (non-poisson), new set to default (poisson)
     if shadowFilteringOld != 4 and shadowFilteringNew == 2:
         log.info('Refactoring global shadow filtering type setting')
 
-        # 0-1 -> 0-1: basic, bilinear -> as is
-        # 2-3 -> 2: pcf, pcf soft -> pcf poisson
-        # 4-5 -> 2-3: pcf poisson, esm -> as is (-2)
         if shadowFilteringOld < 2:
             shadowFilteringNew = shadowFilteringOld
         elif shadowFilteringOld >= 2 and shadowFilteringOld < 4:
@@ -249,7 +246,6 @@ def createSettingsNode():
         cmds.setAttr('v3dSettings.shadowFilteringType', 4) # to default
         cmds.setAttr('v3dSettings.shadowFiltering', shadowFilteringNew)
 
-
 def removeSettingsNode():
     """
     remove v3dSettings node if any
@@ -307,7 +303,6 @@ def parseSettings(output={}):
         elif name == 'shadowFiltering':
             value = SHADOW_FILTERING_MAP[value]
 
-        # color attributes return one-element list with tuple
         elif cmds.attributeQuery(name, node='v3dSettings', usedAsColor=True):
             value = list(value[0])
 
@@ -392,8 +387,6 @@ def parseAnimationSettings(mNode):
 
     output = {}
 
-    # use None for simplicity, but we might want some default values later
-
     defaultSettings = {
         'animAuto': True,
         'animLoop': None,
@@ -453,7 +446,6 @@ def parseAdvRenderSettings(mNode):
 
     return output
 
-
 def assignCameraSettings():
     if cmds.attributeQuery('v3d', type='camera', exists=True):
         return
@@ -510,8 +502,6 @@ def parseCameraSettings(mNode):
 
     output = {}
 
-    # use None for simplicity, but we might want some default values later
-
     defaultSettings = {
         'panningEnabled': True,
         'rotateSpeed': None,
@@ -596,7 +586,6 @@ def parseMeshSettings(mNode):
 
 def assignLightSettings(nodeType):
 
-    # e.g. no Arnold installed/loaded
     if not (nodeType in cmds.listNodeTypes('light')):
         return
 
@@ -686,7 +675,6 @@ def parseLineSettings(mNode):
     }
 
     for name in defaultSettings:
-        # NOTE: handle attr names with suffixes first, e.g lineWidthV3D
         if cmds.attributeQuery(name + 'V3D', node=mNode, exists=True):
             nameAttr = name + 'V3D'
         else:
@@ -694,7 +682,6 @@ def parseLineSettings(mNode):
 
         value = cmds.getAttr(mNode + '.v3d.' + nameAttr)
 
-        # color attributes return one-element list with tuple
         if cmds.attributeQuery(nameAttr, node=mNode, usedAsColor=True):
             value = list(value[0])
 
@@ -704,14 +691,13 @@ def parseLineSettings(mNode):
 
 def assignMaterialSettings(nodeType):
 
-    # e.g. no Arnold installed/loaded
     if not (nodeType in cmds.listNodeTypes('shader')):
         return
 
     if cmds.attributeQuery('v3d', type=nodeType, exists=True):
         return
 
-    numSettings = 6 if (nodeType in MATERIALS_WITH_GLTF_COMPAT) else 5
+    numSettings = 7 if (nodeType in MATERIALS_WITH_GLTF_COMPAT) else 6
 
     cmds.addExtension(nodeType=nodeType, longName='v3d', attributeType='compound',
                       numberOfChildren=numSettings, niceName='Verge3D')
@@ -719,6 +705,8 @@ def assignMaterialSettings(nodeType):
     cmds.addExtension(nodeType=nodeType, longName='alphaMode', attributeType='enum',
                       enumName='Auto:Opaque:Blend:Mask:Coverage:Add', parent='v3d',
                       niceName='Alpha Mode')
+    cmds.addExtension(nodeType=nodeType, longName='transparencyHack', attributeType='enum',
+                      enumName='None:Nearest Layer:Two-Pass', parent='v3d', niceName='Transparency Hack')
     cmds.addExtension(nodeType=nodeType, longName='twoSided', attributeType='bool',
                       defaultValue=False, parent='v3d')
     cmds.addExtension(nodeType=nodeType, longName='depthWrite', attributeType='bool',
@@ -732,7 +720,6 @@ def assignMaterialSettings(nodeType):
         cmds.addExtension(nodeType=nodeType, longName='gltfCompat', attributeType='bool',
                           defaultValue=False, parent='v3d', niceName='glTF 2.0 Compatible')
 
-
 def checkVergeAttrs(mNode):
     if 'v3d' in cmds.listAttr(mNode):
         return True
@@ -743,11 +730,11 @@ def parseMaterialSettings(mMat):
 
     defaultSettings = {
         'alphaMode': 'AUTO',
+        'transparencyHack': 'NONE',
         'twoSided': False,
         'depthWrite': True,
         'depthTest': True,
         'dithering': False,
-        # for unsupported materials and 'usdPreviewSurface'
         'gltfCompat': True
     }
 
@@ -764,20 +751,24 @@ def parseMaterialSettings(mMat):
 
     for name in defaultSettings:
         if name == 'gltfCompat' and (matType not in MATERIALS_WITH_GLTF_COMPAT):
-            if matType != 'usdPreviewSurface':
+            if matType == 'usdPreviewSurface':
+                output['gltfCompat'] = True
+            else:
                 output['gltfCompat'] = False
+
             continue
 
         value = cmds.getAttr(mMat + '.v3d.' + name)
 
         if name == 'alphaMode':
             value = ALPHA_MODE_MAP[value]
+        elif name == 'transparencyHack':
+            value = TRANSPARENCY_HACK_MAP[value]
 
         output[name] = value
 
     return output
 
-
 def assignTextureSettings():
     if cmds.attributeQuery('v3d', type='file', exists=True):
         return
@@ -791,7 +782,6 @@ def assignTextureSettings():
     cmds.addExtension(nodeType='file', longName='compressionMethod', attributeType='enum',
                       enumName=':'.join(COMPRESSION_METHODS), parent='v3d', niceName='Compression')
 
-
 def parseTextureSettings(mTex):
 
     defaultSettings = {
