@@ -3,7 +3,7 @@
 Plugin Name: Verge3D
 Plugin URI: https://www.soft8soft.com/verge3d
 Description: Verge3D is the most artist-friendly toolkit for creating interactive web-based experiences. It can be used to create product configurators, 3D presentations, online stores, e-learning apps, 3D portfolios, browser games and more.
-Version: 4.10.0
+Version: 4.11.0
 Author: Soft8Soft LLC
 Author URI: https://www.soft8soft.com
 License: GPLv2 or later
@@ -25,7 +25,8 @@ function v3d_add_capability() {
     $role->add_cap('manage_verge3d', true);
 
     $role = get_role('editor');
-    $role->add_cap('manage_verge3d', true);
+    if (isset($role))
+        $role->add_cap('manage_verge3d', true);
 }
 register_activation_hook(__FILE__, 'v3d_add_capability');
 
@@ -34,7 +35,8 @@ function v3d_remove_capability() {
     $role->remove_cap('manage_verge3d', true);
 
     $role = get_role('editor');
-    $role->remove_cap('manage_verge3d', true);
+    if (isset($role))
+        $role->remove_cap('manage_verge3d', true);
 }
 register_deactivation_hook(__FILE__, 'v3d_remove_capability');
 
